function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var fabric = require('fabric');

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var _this = undefined;
var createPolygon = function createPolygon(_ref) {
  var xStart = _ref.xStart,
      yStart = _ref.yStart,
      type = _ref.type,
      size = _ref.size;
  var points = [];

  for (var i = 1; i <= type; i += 1) {
    points.push({
      x: xStart + size * Math.sin((i * 2 * Math.PI - Math.PI) / type),
      y: yStart + size * Math.cos((i * 2 * Math.PI - Math.PI) / type)
    });
  }

  return points;
};
var Polygon = React.forwardRef(function (_ref2, ref) {
  var canvasProps = _ref2.canvasProps,
      children = _ref2.children;

  var _React$useState = React__default.useState(null),
      canvas = _React$useState[0],
      setCanvas = _React$useState[1];

  var _React$useState2 = React__default.useState(false),
      drawMode = _React$useState2[0],
      setDrawMode = _React$useState2[1];

  var id = (new Date().getTime() + Math.random() * 10000).toString();
  var lineArray = [];
  var pointArray = [];
  var activeLine;
  var activeShape;
  document.addEventListener("keydown", function (event) {
    if (event.keyCode === 46) {
      handleDeletePolygon();
    }

    if (event.keyCode === 78) {
      handleNextPolygon();
    }

    if (event.keyCode === 80) {
      handlePreviousPolygon();
    }
  });

  var polygonPositionHandler = function polygonPositionHandler(index, _dim, _finalMatrix, fabricObject) {
    var x = fabricObject.points[index].x - fabricObject.pathOffset.x;
    var y = fabricObject.points[index].y - fabricObject.pathOffset.y;
    return fabric.fabric.util.transformPoint({
      x: x,
      y: y
    }, fabric.fabric.util.multiplyTransformMatrices(fabricObject.canvas.viewportTransform, fabricObject.calcTransformMatrix()));
  };

  var actionHandler = function actionHandler(_eventData, transform, x, y) {
    var polygon = transform.target;
    var currentControl = polygon.controls[polygon.__corner];
    var mouseLocalPosition = polygon.toLocalPoint(new fabric.fabric.Point(x, y), "center", "center");

    var polygonBaseSize = polygon._getNonTransformedDimensions();

    var size = polygon._getTransformedDimensions(0, 0);

    var xPoint = mouseLocalPosition.x * polygonBaseSize.x / size.x + polygon.pathOffset.x;
    var yPoint = mouseLocalPosition.y * polygonBaseSize.y / size.y + polygon.pathOffset.y;
    var xOCoordsPoint = polygon.oCoords["p" + currentControl.pointIndex].x;
    var yOCoordsPoint = polygon.oCoords["p" + currentControl.pointIndex].y;

    if (xOCoordsPoint > canvas.width || xOCoordsPoint < 0 || yOCoordsPoint > canvas.height || yOCoordsPoint < 0) {
      return false;
    }

    polygon.points[currentControl.pointIndex] = {
      x: xPoint,
      y: yPoint
    };
    return true;
  };

  var anchorWrapper = function anchorWrapper(anchorIndex, fn) {
    return function (eventData, transform, x, y) {
      var fabricObject = transform.target;
      var absolutePoint = fabric.fabric.util.transformPoint({
        x: fabricObject.points[anchorIndex].x - fabricObject.pathOffset.x,
        y: fabricObject.points[anchorIndex].y - fabricObject.pathOffset.y
      }, fabricObject.calcTransformMatrix());
      var actionPerformed = fn(eventData, transform, x, y);

      fabricObject._setPositionDimensions({});

      var polygonBaseSize = fabricObject._getNonTransformedDimensions();

      var newX = (fabricObject.points[anchorIndex].x - fabricObject.pathOffset.x) / polygonBaseSize.x;
      var newY = (fabricObject.points[anchorIndex].y - fabricObject.pathOffset.y) / polygonBaseSize.y;
      fabricObject.setPositionByOrigin(absolutePoint, newX + 0.5, newY + 0.5);
      return actionPerformed;
    };
  };

  var resizePolygon = function resizePolygon(evt) {
    var poly = evt.target;
    poly.edit = false;
    poly.objectCaching = false;
    poly.controls = fabric.fabric.Object.prototype.controls;
    poly.cornerStyle = "rect";
    poly.hasBorders = true;
    canvas.requestRenderAll();
  };

  var editPolygon = function editPolygon(evt) {
    var poly = evt.target;
    var lastControl = poly.points.length - 1;
    poly.edit = true;
    poly.objectCaching = false;
    poly.cornerStyle = "circle";
    poly.cornerColor = "rgba(0,0,255,0.5)";
    poly.controls = poly.points.reduce(function (acc, _point, index) {
      acc["p" + index] = new fabric.fabric.Control({
        positionHandler: function positionHandler(dim, finalMatrix, fabricObject) {
          return polygonPositionHandler(index, dim, finalMatrix, fabricObject);
        },
        actionHandler: anchorWrapper(index > 0 ? index - 1 : lastControl, actionHandler),
        actionName: "modifyPolygon",
        pointIndex: index
      });
      return acc;
    }, {});
    poly.hasBorders = false;
    canvas.requestRenderAll();
  };

  var addAndSetEventPolygon = function addAndSetEventPolygon(points) {
    var options = _extends({
      fill: canvasProps.canEdit === false ? "rgba(0,0,0,0)" : "rgba(208,201,203,0.5)",
      strokeWidth: 1,
      stroke: "green",
      scaleX: 1,
      scaleY: 1,
      objectCaching: false,
      transparentCorners: false,
      cornerColor: "blue",
      cornerStyle: "circle",
      cornerSize: 10
    }, canvasProps.polygonStyle);

    var polygon = new fabric.fabric.Polygon(points, options);
    canvas.add(polygon);
    polygon.on("selected", editPolygon);
    polygon.on("mousedblclick", resizePolygon);
  };

  var handleAddPolygon = function handleAddPolygon(type, size) {
    if (canvas) {
      var points = createPolygon({
        xStart: canvasProps.width / 2,
        yStart: canvasProps.height / 2,
        type: type,
        size: size
      });
      addAndSetEventPolygon(points);
    }
  };

  var handleAddAndReplacePolygon = function handleAddAndReplacePolygon(id, points) {
    if (canvas) {
      canvas.getObjects().forEach(function (item) {
        if (item.hasOwnProperty("id") && item.get("id") === id) {
          canvas.remove(item);
        }
      });
      var polygon = new fabric.fabric.Polygon(points, {
        fill: "rgba(0,0,0,0)",
        strokeWidth: 1,
        stroke: "red",
        scaleX: 1,
        scaleY: 1,
        objectCaching: false,
        transparentCorners: false,
        id: id
      });
      canvas.add(polygon);
      polygon.on("selected", editPolygon);
      polygon.on("mousedblclick", resizePolygon);
      canvas.requestRenderAll();
    }
  };

  var handleConfirmUpdate = function handleConfirmUpdate() {
    var polygonList = [];

    if (canvas) {
      var _loop = function _loop(i) {
        var oCoords = canvas._objects[i].oCoords;
        var polygon = [];
        Object.keys(oCoords).forEach(function (key) {
          if (key.charAt(0) === "p") {
            polygon.push({
              x: oCoords[key].x < 0 ? 0 : oCoords[key].x,
              y: oCoords[key].y < 0 ? 0 : oCoords[key].y
            });
          }
        });

        if (polygon.length > 0) {
          polygonList.push({
            polygon: polygon,
            label: canvas._objects[i].label ? canvas._objects[i].label : "Label " + (i + 1)
          });
        } else {
          var points = canvas._objects[i].points;

          if (points.length > 0) {
            polygonList.push({
              polygon: points,
              label: canvas._objects[i].label ? canvas._objects[i].label : "Label " + (i + 1)
            });
          }
        }
      };

      for (var i = 0; i < canvas._objects.length; i++) {
        _loop(i);
      }

      canvas.requestRenderAll();
    }

    return polygonList;
  };

  var handleDeletePolygon = function handleDeletePolygon() {
    if (canvas) {
      var activeObject = canvas.getActiveObject();
      canvas.remove(activeObject);
      canvas.requestRenderAll();
    }
  };

  var handleNextPolygon = function handleNextPolygon() {
    if (canvas) {
      var activeObject = canvas.getActiveObject();

      if (!activeObject) {
        activeObject = canvas.getObjects()[0];
        canvas.setActiveObject(activeObject);
      } else {
        for (var i = 0; i < canvas._objects.length; i++) {
          if (canvas._objects[i] === activeObject) {
            canvas.setActiveObject(i > canvas._objects.length - 2 ? canvas._objects[0] : canvas._objects[i + 1]);
            break;
          }
        }
      }
    }
  };

  var handlePreviousPolygon = function handlePreviousPolygon() {
    if (canvas) {
      var activeObject = canvas.getActiveObject();

      if (!activeObject) {
        activeObject = canvas.getObjects()[0];
        canvas.setActiveObject(activeObject);
      } else {
        for (var i = 0; i < canvas._objects.length; i++) {
          if (canvas._objects[i] === activeObject) {
            canvas.setActiveObject(i === 0 ? canvas._objects[canvas._objects.length - 1] : canvas._objects[i - 1]);
            break;
          }
        }
      }
    }
  };

  var handleMoving = function handleMoving(obj) {
    obj.objectCaching = false;

    if (obj.currentHeight > obj.canvas.height || obj.currentWidth > obj.canvas.width) {
      return;
    }

    obj.setCoords();

    if (obj.getBoundingRect().top < 0 || obj.getBoundingRect().left < 0) {
      obj.top = Math.max(obj.top, obj.top - obj.getBoundingRect().top);
      obj.left = Math.max(obj.left, obj.left - obj.getBoundingRect().left);
    }

    if (obj.getBoundingRect().top + obj.getBoundingRect().height > obj.canvas.height || obj.getBoundingRect().left + obj.getBoundingRect().width > obj.canvas.width) {
      obj.top = Math.min(obj.top, obj.canvas.height - obj.getBoundingRect().height + obj.top - obj.getBoundingRect().top);
      obj.left = Math.min(obj.left, obj.canvas.width - obj.getBoundingRect().width + obj.left - obj.getBoundingRect().left);
    }

    canvas.requestRenderAll();
  };

  var toggleDrawPolygon = function toggleDrawPolygon() {
    if (drawMode) {
      activeLine = null;
      activeShape = null;
      lineArray = [];
      pointArray = [];
      canvas.selection = true;
      setDrawMode(false);
    } else {
      canvas.selection = false;
      setDrawMode(true);
    }
  };

  var addPoint = function addPoint(options) {
    var pointOption = {
      id: new Date().getTime(),
      radius: 5,
      fill: "#ffffff",
      stroke: "#333333",
      strokeWidth: 0.5,
      left: options.e.layerX / canvas.getZoom(),
      top: options.e.layerY / canvas.getZoom(),
      selectable: false,
      hasBorders: false,
      hasControls: false,
      originX: "center",
      originY: "center",
      objectCaching: false
    };
    var point = new fabric.fabric.Circle(pointOption);

    if (pointArray.length === 0) {
      point.set({
        fill: "red"
      });
    }

    var linePoints = [options.e.layerX / canvas.getZoom(), options.e.layerY / canvas.getZoom(), options.e.layerX / canvas.getZoom(), options.e.layerY / canvas.getZoom()];
    var lineOption = {
      strokeWidth: 2,
      fill: "#999999",
      stroke: "#999999",
      originX: "center",
      originY: "center",
      selectable: false,
      hasBorders: false,
      hasControls: false,
      evented: false,
      objectCaching: false
    };
    var line = new fabric.fabric.Line(linePoints, lineOption);
    line["class"] = "line";

    if (activeShape) {
      var pos = canvas.getPointer(options.e);
      var points = activeShape.get("points");
      points.push({
        x: pos.x,
        y: pos.y
      });
      var polygon = new fabric.fabric.Polygon(points, {
        stroke: "#333333",
        strokeWidth: 1,
        fill: "#cccccc",
        opacity: 0.3,
        selectable: false,
        hasBorders: false,
        hasControls: false,
        evented: false,
        objectCaching: false
      });
      canvas.remove(activeShape);
      canvas.add(polygon);
      activeShape = polygon;
      canvas.renderAll();
    } else {
      var polyPoint = [{
        x: options.e.layerX / canvas.getZoom(),
        y: options.e.layerY / canvas.getZoom()
      }];

      var _polygon = new fabric.fabric.Polygon(polyPoint, {
        stroke: "#333333",
        strokeWidth: 1,
        fill: "#cccccc",
        opacity: 0.3,
        selectable: false,
        hasBorders: false,
        hasControls: false,
        evented: false,
        objectCaching: false
      });

      activeShape = _polygon;
      canvas.add(_polygon);
    }

    activeLine = line;
    pointArray.push(point);
    lineArray.push(line);
    canvas.add(line);
    canvas.add(point);
  };

  var generatePolygonFromFreeDraw = function generatePolygonFromFreeDraw(pointArray) {
    var points = [];

    for (var _iterator = _createForOfIteratorHelperLoose(pointArray), _step; !(_step = _iterator()).done;) {
      var point = _step.value;
      points.push({
        x: point.left,
        y: point.top
      });
      canvas.remove(point);
    }

    for (var _iterator2 = _createForOfIteratorHelperLoose(lineArray), _step2; !(_step2 = _iterator2()).done;) {
      var line = _step2.value;
      canvas.remove(line);
    }

    canvas.remove(activeShape).remove(activeLine);
    addAndSetEventPolygon(points);
    toggleDrawPolygon();
  };

  var onMouseDown = function onMouseDown(options) {
    if (drawMode) {
      if (options.target && options.target.id === pointArray[0].id) {
        generatePolygonFromFreeDraw(pointArray);
      } else {
        addPoint(options);
      }
    }

    var evt = options.e;

    if (evt.altKey === true) {
      _this.lastPosX = evt.clientX;
    }
  };

  var onMouseMove = function onMouseMove(options) {
    if (drawMode) {
      if (activeLine && activeLine["class"] === "line") {
        var pointer = canvas.getPointer(options.e);
        activeLine.set({
          x2: pointer.x,
          y2: pointer.y
        });
        var points = activeShape.get("points");
        points[pointArray.length] = {
          x: pointer.x,
          y: pointer.y
        };
        activeShape.set({
          points: points
        });
      }

      canvas.renderAll();
    }
  };

  var _React$useState3 = React__default.useState(null),
      outerCanvasContainer = _React$useState3[0],
      setCanvasContainer = _React$useState3[1];

  React.useEffect(function () {
    var newCanvas = new fabric.fabric.Canvas(id, {
      width: canvasProps.width,
      height: canvasProps.height
    });
    var tmp = document.getElementById(id + "_container");

    if (tmp) {
      setCanvasContainer(tmp);
      var ratio = newCanvas.getWidth() / newCanvas.getHeight();
      var containerWidth = tmp.clientWidth;
      var scale = containerWidth / newCanvas.getWidth();
      var zoom = newCanvas.getZoom() * scale;
      newCanvas.setDimensions({
        width: containerWidth,
        height: containerWidth / ratio
      });
      newCanvas.setViewportTransform([zoom, 0, 0, zoom, 0, 0]);
    }

    setCanvas(newCanvas);
    fabric.fabric.Image.fromURL(canvasProps.backgroundImage, function (myImg) {
      var img = myImg.set({
        left: 0,
        top: 0,
        width: newCanvas.width,
        height: newCanvas.height
      });
      newCanvas.setBackgroundImage(img, newCanvas.renderAll.bind(newCanvas), {
        scaleX: newCanvas.width / img.width,
        scaleY: newCanvas.height / img.height
      });
    });
  }, []);
  React.useEffect(function () {
    if (canvas) {
      new ResizeObserver(resizeCanvas).observe(outerCanvasContainer);

      if (canvasProps.defaultPolygons) {
        var polygons = canvasProps.defaultPolygons;

        for (var i = 0; i < polygons.length; i++) {
          var polygon = polygons[i].polygon;
          addAndSetEventPolygon(polygon);
        }
      }

      canvas.on("object:moving", function (e) {
        return handleMoving(e.target);
      });
      canvas.on("object:modified", function (e) {
        return handleMoving(e.target);
      });
    }
  }, [canvas]);
  React.useEffect(function () {
    if (canvas && drawMode) {
      canvas.off("mouse:down");
      canvas.off("mouse:move");
      canvas.on("mouse:down", onMouseDown);
      canvas.on("mouse:move", onMouseMove);
    }
  }, [drawMode]);
  React.useImperativeHandle(ref, function () {
    return {
      toggleDraw: function toggleDraw() {
        toggleDrawPolygon();
      },
      onDraw: function onDraw(option) {
        handleAddPolygon(option.type, option.size);
      },
      onDrawAndReplace: function onDrawAndReplace(option) {
        handleAddAndReplacePolygon(option.id, option.points);
      },
      onConfirm: function onConfirm() {
        return handleConfirmUpdate();
      }
    };
  });

  var resizeCanvas = function resizeCanvas() {
    if (outerCanvasContainer) {
      var ratio = canvas.getWidth() / canvas.getHeight();
      var containerWidth = outerCanvasContainer.clientWidth;
      var scale = containerWidth / canvas.getWidth();
      var zoom = canvas.getZoom() * scale;
      canvas.setDimensions({
        width: containerWidth,
        height: containerWidth / ratio
      });
      canvas.setViewportTransform([zoom, 0, 0, zoom, 0, 0]);
    }
  };

  return React__default.createElement("div", {
    id: id + "_container",
    style: {
      position: "relative"
    }
  }, React__default.createElement("div", {
    style: {
      position: "absolute",
      top: 0,
      left: 0,
      right: 0,
      bottom: 0
    }
  }, children), React__default.createElement("canvas", {
    id: id
  }), canvasProps.canEdit === false && React__default.createElement("div", {
    style: {
      position: "absolute",
      top: 0,
      left: 0,
      right: 0,
      bottom: 0
    }
  }));
});

var POLYGON_SIZE = {
  small: 50,
  normal: 100,
  large: 150
};
var POLYGON_TYPE = {
  rec: 4,
  hex: 6,
  oct: 8
};
var CanvasPolygons = React.forwardRef(function (_ref, ref) {
  var children = _ref.children,
      canvasHeight = _ref.canvasHeight,
      canvasWidth = _ref.canvasWidth,
      defaultPolygons = _ref.defaultPolygons,
      canEdit = _ref.canEdit,
      polygonStyle = _ref.polygonStyle;
  var polygonRef = React__default.useRef();
  React.useImperativeHandle(ref, function () {
    return {
      toggleDraw: function toggleDraw() {
        polygonRef.current.toggleDraw();
      },
      onDraw: function onDraw(option) {
        polygonRef.current.onDraw(option);
      },
      onDrawAndReplace: function onDrawAndReplace(option) {
        polygonRef.current.onDrawAndReplace(option);
      },
      onConfirm: function onConfirm() {
        return polygonRef.current.onConfirm();
      }
    };
  });
  return React__default.createElement(Polygon, {
    ref: polygonRef,
    canvasProps: {
      width: canvasWidth,
      height: canvasHeight,
      defaultPolygons: defaultPolygons,
      canEdit: canEdit,
      polygonStyle: polygonStyle
    }
  }, children);
});

exports.POLYGON_SIZE = POLYGON_SIZE;
exports.POLYGON_TYPE = POLYGON_TYPE;
exports.default = CanvasPolygons;
//# sourceMappingURL=index.js.map
